// **********************************************************************
//
// Copyright (c) 1998
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// Copyright (c) 1998
// Object-Oriented Concepts GmbH
// Ettlingen, Germany
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_PROPERTIES_H
#define OB_PROPERTIES_H

namespace OB
{

class Properties
{
    //
    // Hide copy-constructor and assignment operator
    //
    Properties(const Properties&);
    void operator=(const Properties&);

protected:

    //
    // The Properties singleton
    //
    static OB_IMPORT Properties* instance_;
    friend class PropertiesDestroyer;

    //
    // Properties data
    //
    StringSeq props_;
    
    //
    // Load the properties from a file
    //
    void load(const char*);

#ifdef WIN32
    //
    // Load the properties from the registry.
    //
    void load(HKEY, const char*, const char*);
#endif

    Properties();

public:

    ~Properties();
    
    typedef StringSeq KeySeq;

    //
    // Set a property
    //
    void setProperty(const char*, const char*);

    //
    // Get property
    //
    const char* getProperty(const char*) const;

    //
    // Get all property keys starting with prefix
    //
    KeySeq getKeys(const char* prefix) const;

    //
    // Get all property keys
    //
    KeySeq getKeys() const;

    //
    // Get the properties singleton (which may be null, if the
    // properties have not yet been initialized)
    //
    static Properties* instance();

    //
    // Initialize the properties
    //
#ifdef WIN32
    static Properties* init(int&, char**, bool = true);
#else
    static Properties* init(int&, char**);
#endif
};

} // End of namespace OB

#endif
